model traffic

global {
	graph road_graph;
	float step <- 1#mn;
	map speed_map update: road as_map (each::each.shape.perimeter / each.speed_coeff);

	shape_file buildings_shapefile <- shape_file("../includes/buildings.shp");
	shape_file roads_shapefile <- shape_file("../includes/roads.shp");

	geometry shape <- envelope(buildings_shapefile);

	init {
		create building from: buildings_shapefile;
		create road from: roads_shapefile;
		road_graph <- as_edge_graph(road);
		create people number: 1000 ;
		speed_map <- road as_map (each::each.shape.perimeter / each.speed_coeff);
	}
}

species road {
	int nb_people update: length(people at_distance 0.1);
	int capacity <- int(shape.perimeter/30.0) min: 1;
	float speed_coeff <- 1.0 update: nb_people = 0 ? 1 : capacity/nb_people min: 0.3 max: 1.0;

	aspect default {
		draw shape + 5 color: rgb(0, 0, 0);
	}

	aspect road_use {
		draw shape + 1 + nb_people color: rgb(255, 0, 128);
	}
}

species people skills:[moving] {
	point target;
	float speed <- 5#km/#h;
	float proba_stay <- 0.05;
	
	init {
		location <- any_location_in(one_of(building));
	}
	
	reflex move when: target != nil {
		do goto target: target on: road_graph move_weights: speed_map;
		if (location = target) {
		     target <- nil;
		}
	}
	
	reflex choose_target when: target = nil and flip(proba_stay) {
		target <- any_location_in(one_of(building));
	}
	
	aspect Geom {
		draw cube(10) color: rgb(0, 0, 255);
	}
}

species building {
	aspect default {
		draw shape color: rgb(0, 0, 255) depth: 10 + rnd(20) 
				texture: ["../includes/roof_top.png","../includes/texture5.jpg"];
	}
}


experiment my_GUI_xp type:gui {
	output{
		display map type: opengl {
			image"../includes/soil.jpg" color:rgb(255,255,255);
			species building refresh:false;
			species road refresh:false;
			species people aspect: Geom;
		}
		
		display road_use {
			species road aspect: road_use;
		}
	}
}
